#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import pkg_resources

# import interfaces
from pyams_utils.interfaces.site import ISiteRoot
from pyams_utils.interfaces.tales import ITALESExtension
from pyams_zmi.interfaces import IZMIConfiguration

# import packages
from persistent import Persistent
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter, get_annotation_adapter
from pyams_utils.traversing import get_parent
from zope.container.contained import Contained
from zope.interface import implementer, Interface
from zope.schema.fieldproperty import FieldProperty


@implementer(IZMIConfiguration)
class ZMIConfiguration(Persistent, Contained):
    """ZMI configuration class"""

    application_package = FieldProperty(IZMIConfiguration['application_package'])
    application_name = FieldProperty(IZMIConfiguration['application_name'])
    inner_package = FieldProperty(IZMIConfiguration['inner_package'])
    inner_package_name = FieldProperty(IZMIConfiguration['inner_package_name'])
    version_location = FieldProperty(IZMIConfiguration['version_location'])

    include_header = FieldProperty(IZMIConfiguration['include_header'])
    fixed_header = FieldProperty(IZMIConfiguration['fixed_header'])
    include_top_links = FieldProperty(IZMIConfiguration['include_top_links'])

    include_site_search = FieldProperty(IZMIConfiguration['include_site_search'])
    site_search_placeholder = FieldProperty(IZMIConfiguration['site_search_placeholder'])
    site_search_handler = FieldProperty(IZMIConfiguration['site_search_handler'])
    include_mobile_search = FieldProperty(IZMIConfiguration['include_mobile_search'])
    mobile_search_placeholder = FieldProperty(IZMIConfiguration['mobile_search_placeholder'])
    mobile_search_handler = FieldProperty(IZMIConfiguration['mobile_search_handler'])

    include_user_activity = FieldProperty(IZMIConfiguration['include_user_activity'])
    include_user_shortcuts = FieldProperty(IZMIConfiguration['include_user_shortcuts'])
    include_logout_button = FieldProperty(IZMIConfiguration['include_logout_button'])
    include_minify_button = FieldProperty(IZMIConfiguration['include_minify_button'])
    include_flags = FieldProperty(IZMIConfiguration['include_flags'])

    include_menus = FieldProperty(IZMIConfiguration['include_menus'])
    fixed_navigation = FieldProperty(IZMIConfiguration['fixed_navigation'])
    accordion_menus = FieldProperty(IZMIConfiguration['accordion_menus'])

    include_ribbon = FieldProperty(IZMIConfiguration['include_ribbon'])
    fixed_ribbon = FieldProperty(IZMIConfiguration['fixed_ribbon'])
    include_reload_button = FieldProperty(IZMIConfiguration['include_reload_button'])

    base_body_css_class = FieldProperty(IZMIConfiguration['base_body_css_class'])

    @property
    def version(self):
        result = pkg_resources.get_distribution(self.application_package).version
        if self.inner_package and (self.inner_package != self.application_package):
            result += ' (%s v%s)' % (self.inner_package_name,
                                     pkg_resources.get_distribution(self.inner_package).version)
        return result

    @property
    def body_css_class(self):
        result = self.base_body_css_class or ''
        for attr in ('header', 'ribbon', 'navigation'):
            if getattr(self, 'fixed_' + attr, False):
                result += ' fixed-' + attr
        return result


ZMI_CONFIGURATION_KEY = 'pyams_zmi.configuration'


@adapter_config(context=ISiteRoot, provides=IZMIConfiguration)
def zmi_configuration_factory(context):
    """ZMI configuration factory"""
    return get_annotation_adapter(context, ZMI_CONFIGURATION_KEY, ZMIConfiguration,
                                  notify=False)


@adapter_config(name='zmi_configuration', context=(Interface, Interface, Interface), provides=ITALESExtension)
class ZMIConfigurationTalesExtension(ContextRequestViewAdapter):
    """extension:zmi_configuration TALES expression"""

    def render(self, context=None):
        if context is None:
            context = self.context
        target = get_parent(context, ISiteRoot)
        return IZMIConfiguration(target)
